
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the VRLIB_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// VRLIB_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef VRLIB_EXPORTS
#define VRLIB_API __declspec(dllexport)
#else
#define VRLIB_API __declspec(dllimport)
#endif

#include <wtypes.h>

extern "C" {
#include "hidsdi.h"
#include <setupapi.h>
}

// VID and PID
#define RADIATOR_VID	0x1527
#define RADIATOR_PID	0x0301

// Choices for FreqAndHex function
#define FREQTOHEX	1
#define HEXTOFREQ	2

// Number of bytes for buffers
#define FREQ_BYTE	2
#define DATA_BYTE	9

// Hardware Protocol

// Structure of SendData (Host to Radio)
// 0 -> Buffer				-> Device does not take in this byte
// 1 -> PacketID			-> 0xB1
// 2 -> GPIO Ctrl			-> 00-> Led Off (Not used anymore)
//							   01-> Led On  (Not used anymore)
//							   02-> Host reading freq bytes from radio chip
//							   03-> Host sending data to radio chip
//							   04-> Radio Mute
//							   05-> Radio Unmute
//							   06-> Firmware Version
//							   07-> Set LED to RED (When power on but app not launched)
//							   08-> Set LED to GREEN (When app is launched)
//							   09-> Set LED to blinking GREEN (When recording is in process)
// 3 -> Tuning Code Byte 1	-> 1/0 Indicate Search start/end
// 4 -> Tuning Code Byte 2	-> Contains search up/down, Stereo/Mono, Band, Locking field strength
// 5 -> Tuning Code Byte 3	-> Frequency byte1
// 6 -> Tuning Code Byte 4	-> Frequency byte2
// 7 -> Unused				-> 
// 8 -> Unused				-> 

// Structure of RetrievedData (Radio to Host)
// 0 -> Buffer				-> Unused
// 1 -> PacketID			-> 0xB1
// 2 -> LED Status			-> 00-> LED Off, 01-> LED On (Not used anymore)
// 3 -> AuFmt				-> 00-> Stereo
//							   01-> Mono
// 4 -> Tuning Data 1		-> Data
// 5 -> Tuning Data 2		-> Data
// 6 -> Tuning Data 3		-> Data / Frequency byte1 if GPREADFREQ was sent previously
// 7 -> Tuning Data 4		-> Data / Frequency byte2 if GPREADFREQ was sent previously
// 8 -> IR Data				-> IR code received

// Defining index of data in SendData
#define SDPACKETID	1
#define SDGPIOCTRL	2
#define SDTCBYTE_1	3
#define SDTCBYTE_2	4
#define SDTCBYTE_3	5
#define SDTCBYTE_4	6

// Defining index of data in RetrievedData
#define RDPACKETID	1
#define RDLEDSTATUS	2
#define RDTUNESTATE	3
#define RDTCBYTE_1	4
#define RDTCBYTE_2	5
#define RDTCBYTE_3	6
#define RDTCBYTE_4	7
#define	RDIRCODE	8

// Definitions of data constants

typedef enum
{
	GPLEDOFF		= 0,
	GPLEDON			= 1,
	GPREADFREQ		= 2,
	GPSENDDATA		= 3,
	GPMUTE			= 4,
	GPUNMUTE		= 5,
	GPFIRMVER		= 6,
	GPLEDRED		= 7,
	GPLEDGREEN		= 8,
	GPLEDRECSTATE	= 9,
} GPIOCTRL;	// GPIO control byte

typedef enum
{
	START	= 0,
	END		= 1,
} SR_STATUS;	// Search Status

typedef enum
{
	UP		= 0,
	DOWN	= 1,
} SR_DIRECTN;	// Search Direction

typedef enum
{
	MONO	= 0,
	STEREO	= 1,
} AUDIO_FORMAT;

typedef enum 
{
	AM_BAND		= 0,
	FM_BAND		= 1,
} TUNE_BAND;

// Not used anymore
/*typedef enum
{
	LEFT		= 0,
	RIGHT		= 1,
	CENTER		= 2,
} BALANCE;*/

// Definition of control data

// Define default packet ID
#define PACKETID	0xB1	

// Define SendData GPIO control bits
#define GPIO_LEDOFF			0x00
#define GPIO_LEDON			0x01
#define GPIO_READFREQ		0x02
#define GPIO_SENDDATA		0x03
#define GPIO_MUTE			0x04
#define GPIO_UNMUTE			0x05
#define GPIO_FIRMVER		0x06
#define GPIO_LEDRED			0x07
#define GPIO_LEDGREEN		0x08
#define GPIO_LEDRECSTATE	0x09

// Hexadecimal value to control start and end in Tuning Code Byte 1
#define STARTBIT	0x01	// 0000 0001 (to be ORed)
#define ENDBIT		0xFE	// 1111 1110 (to be ANDed)

// Hexadecimal value to control start and end in bit 7 of Tuning Code Byte 2
#define UPBIT		0x80	// 1000 0000 (to be ORed)
#define DOWNBIT		0x7F	// 0111	1111 (to be ANDed)

// Hexadecimal value to control mono and stereo in bit 6 of Tuning Code Byte 2
#define MONOBIT		0x40	// 0100 0000 (to be ORed)
#define STEREOBIT	0xBF	// 1011 1111 (to be ANDed)

// Hexadecimal value to control AM and FM in bits 4 and 5 of Tuning Code Byte 2
// In our program, we are using FM only. The AM band is set assuming it is LW band.
#define AMBIT		0x20	// 0010 0000 (to be ORed)
#define FMBIT		0xCF	// 1100 1111 (to be ANDed)

// Hexadecimal value to control programmable ports P0 and P1. They are not used in
// our hardware device. Bits 2 and 3 of Tuning Code Byte 2 are set to zero.
#define PPORTBIT	0xF3	// 1111 0011 (to be ANDed)

// Hexadecimal value to control searching field strength.
#define SEARCHFIELDSTR1	0x00	// 0000 0000 (to be ORed)
#define SEARCHFIELDSTR2	0x02	// 0000 0010 (to be ORed)
#define SEARCHFIELDSTR3	0x01	// 0000 0001 (to be ORed)
#define SEARCHFIELDSTR4	0x03	// 0000 0011 (to be ORed)

// Hexadecimal value for ReceivedData LED Status bits
#define LEDSTATUS_OFF	0x00
#define LEDSTATUS_ON	0x01

// Define the FM band frequency range
#define FMLOW	8700	// Frequency in 10kHz
#define FMHIGH	10800	// Frequency in 10kHz

// Define DLL Version
#define DLL_VERMAJ		1
#define DLL_VERMED		8
#define DLL_VERMIN		3


// This class is exported from the VRLib.dll
class VRLIB_API CVRLib {
public:
	CVRLib();
	// Init / Deinit functions
	BOOL Initialize();									// Returns TRUE if radio device is found
	VOID Shutdown();									// Perform shutdown routine.
	// Tuner functions
	HRESULT		SetGPIOCTRL(GPIOCTRL Control);			// Set the GPIOCTRL byte. Refer to table above
	HRESULT		SetSearchFieldStrength(int strength);	// Set the searching field strength for radio
	HRESULT		SetSearchStatus(SR_STATUS SrStatus);	// Start / End
	HRESULT		SetSearchDirection(SR_DIRECTN SrDrtn);	// Up / Down
	HRESULT		SetAuFormat(AUDIO_FORMAT AuFormat);		// stereo / mono
	AUDIO_FORMAT GetAuFormat();							// Returns an Audio Format
	VOID		SetPredefinedByte();					// Set Programmable ports and searching field bits
	VOID		SetTuneRange();							// Set the tunning range for FM band
	HRESULT		SetFreqInSendData(DWORD dwFreqHz);		// Set freq bytes in SendData
	HRESULT		SetTuneFreq(DWORD dwFreqHz);			// Frequency in 10kHz
	DWORD		GetTuneFreq();							// Returns the current tuned frequency
	DWORD		StartSearch(SR_DIRECTN SrDrtn, DWORD dwCurFreq);	// Start the searching algorithm
	HRESULT		MuteVolume();							// Hardware Mute
	HRESULT		UnMuteVolume();							// Hardware UnMute
	HRESULT		SetLEDRed();							// Set LED color to red
	HRESULT		SetLEDGreen();							// Set LED color to green
	HRESULT		SetLEDRecState();						// Set LED color to recording state
	DWORD		GetIRCode();							// Returns last IR code received
	HRESULT		GetFirmwareVer(DWORD* dwFmVer1, DWORD* FmVer2);		// Return the Firmware Version
	HRESULT		GetDLLVer(INT* iDllVer1, INT* iDllVer2, INT* iDllVer3);	// Return the DLL Version
	// Calculation Functions
	VOID		StrtoHex(char *inStr, DWORD *dwData);	// Convert a string to hex values
	VOID		FreqAndHex(DWORD* theFreq, int choice);	// Convert to freq in 10kHz or hex
		
private:
	// Array to store data for sending to device
	UINT	m_uiSendData[DATA_BYTE];		// Array to store data to be sent to radio chip in int format
	CHAR	m_cActualSendData[DATA_BYTE];	// Array to send data in char format
	CHAR	m_cRetrievedData[DATA_BYTE];	// Array to store data retrieved from radio chip
	DWORD	m_dwBandMinFreq;				// A variable to store the min freq for the band
	DWORD	m_dwBandMaxFreq;				// A variable to store the min freq for the band
	BOOL	m_bBandLoop;					// Returns TRUE if search has reached the end of range
	BOOL	m_bTuneStatus;					// Returns TRUE if a tuned state is detected
	// Variables specially for USB communication
	DWORD		NumberOfBytesRead;		// Number of bytes read using the ReadFile function
	BOOL		DeviceDetected;			// TRUE when HID device is detected.
	HANDLE		DeviceHandle;			// Handle to a HID device
	HANDLE		hEventObject;			// Handle to event object used in OVERLAPPED structure
	HANDLE		hDevInfo;				// HAndle to contain information set of all installed devices
	HANDLE		ReadHandle;				// Handle created for reading overlapped data
	GUID		HidGuid;				// Variable to store all avialble GUIDs of system HIDs
	OVERLAPPED	HIDOverlapped;			// Structure to be used in ReadFile() function
	HIDP_CAPS	Capabilities;			// Data structure to store device's capablilities
	PSP_DEVICE_INTERFACE_DETAIL_DATA	detailData;	// A structure that contains the path for a device interface
	// USB Communication functions
	BOOL	FindTheHID();						// Find our HID device and establish connection
	VOID	GetDeviceCapabilities();			// Get device's capablities
	VOID	PrepareForOverlappedTransfer();		// Get handle for overlapped transfer
	HRESULT	SendDataToDevice();					// Prepare data to send to device
	HRESULT WriteReport();						// Send data to HID device
	HRESULT ReadReport();						// Read data retrieved at HID device	
};

extern VRLIB_API int nVRLib;

VRLIB_API int fnVRLib(void);

// For external programmer
extern "C"
{
	VRLIB_API CVRLib* pLib;
	VRLIB_API BOOL Initialize();
	VRLIB_API VOID Shutdown();
	VRLIB_API HRESULT SetAuFormat(AUDIO_FORMAT AuFormat);
	VRLIB_API AUDIO_FORMAT GetAuFormat();
	VRLIB_API HRESULT SetTuneFreq(DWORD dwFreq);
	VRLIB_API DWORD StartSearch(SR_DIRECTN SrDrtn, DWORD dwCurFreq);
	VRLIB_API HRESULT MuteVolume();
	VRLIB_API HRESULT UnMuteVolume();
	VRLIB_API HRESULT SetLEDRecStart();
	VRLIB_API HRESULT SetLEDRecStop();
	VRLIB_API HRESULT SetLEDAppStart();
	VRLIB_API HRESULT SetLEDAppStop();
	VRLIB_API HRESULT GetFirmwareVer(DWORD* dwFmVer1, DWORD* dwFmVer2);
	VRLIB_API HRESULT GetDLLVer(INT* iDllVer1, INT* iDllVer2, INT* iDllVer3);
}
